"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.deserializeErrorMessages = exports.serializeErrorMessages = exports.collectStackErrorMessages = exports.getStatusToErrorMsg = void 0;
const getStatusToErrorMsg = (status) => {
    const MAP = {
        CREATE_FAILED: 'create',
        DELETE_FAILED: 'delete',
        UPDATE_FAILED: 'update',
    };
    return MAP[status] || status;
};
exports.getStatusToErrorMsg = getStatusToErrorMsg;
const collectStackErrorMessages = (eventsWithFailure, customStackIds) => {
    const errorMessages = {
        messages: eventsWithFailure.map((event) => {
            const name = `${event.LogicalResourceId} (${event.ResourceType})`;
            const eventType = `${(0, exports.getStatusToErrorMsg)(event.ResourceStatus)}`;
            const reason = `${event.ResourceStatusReason}`;
            const isCustomResource = customStackIds.includes(event.PhysicalResourceId) || customStackIds.includes(event.StackId);
            const errorMessage = { name, eventType, reason, isCustomResource };
            return errorMessage;
        }),
    };
    return (0, exports.serializeErrorMessages)(errorMessages);
};
exports.collectStackErrorMessages = collectStackErrorMessages;
const serializeErrorMessages = (errorMessages) => {
    const serializedStringParts = [];
    errorMessages.messages.forEach((errorMessage) => {
        let currentString = `Name: ${errorMessage.name}, `;
        currentString += `Event Type: ${errorMessage.eventType}, `;
        currentString += `Reason: ${errorMessage.reason}, `;
        currentString += `IsCustomResource: ${errorMessage.isCustomResource}\n`;
        serializedStringParts.push(currentString);
    });
    return serializedStringParts.join('\n');
};
exports.serializeErrorMessages = serializeErrorMessages;
const deserializeErrorMessages = (errorDetails) => {
    const deserializedMessages = { messages: [] };
    const separateLines = errorDetails === null || errorDetails === void 0 ? void 0 : errorDetails.split('\n');
    separateLines === null || separateLines === void 0 ? void 0 : separateLines.forEach((line) => {
        const separateFields = line.split(/Name: |, Event Type: |, Reason: |, IsCustomResource: /);
        const [, name, eventType, reason, isCustomResourceString] = separateFields;
        const isCustomResource = isCustomResourceString === 'true';
        if (name && eventType && reason) {
            const deserializedMessage = { name, eventType, reason, isCustomResource };
            deserializedMessages.messages.push(deserializedMessage);
        }
    });
    return deserializedMessages;
};
exports.deserializeErrorMessages = deserializeErrorMessages;
//# sourceMappingURL=cloudformation-error-serializer.js.map